__author__ = 'lpreimesberger'
__copyright__ = 'copyright (c) 2015 census protocol foundation'
#
#
# this is part of the test suite - you are free to use  the software for any  purpose
# we welcome your donations - all are tax deductible and  go to education and pay for
# core  protocol  developers and  evangelists.  the sitting board receives no salary.

#  "You  have  to  find  a  way to earn your living without transgressing your ideals
#   of love and compassion" - Thich Nhat Hanh

#                          19MSTA2Zcuga7rTqLY5vfXLWDBkdfcVQ2U
#
#  if you require a tax receipt for your donation - send the txid to donate@censusprotocol.org
#
#
# The MIT License (MIT)
#
# Copyright (c) 2015 Census Protocol Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining a  copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the  rights
# to use, copy, modify, merge, publish,  distribute,  sublicense,  and/or  sell
# copies  of  the  Software,  and  to  permit persons to whom the  Software  is
# furnished to do so, subject to the following conditions:
#
# The  above copyright notice  and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE  SOFTWARE  IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING  BUT  NOT LIMITED  TO  THE  WARRANTIES OF MERCHANTABILITY,
# FITNESS  FOR  A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS   OR  COPYRIGHT  HOLDERS  BE LIABLE  FOR ANY  CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR  OTHERWISE, ARISING FROM,
# OUT OF  OR  IN  CONNECTION WITH THE SOFTWARE OR THE USE  OR OTHER DEALINGS IN
# THE SOFTWARE.


import urllib2
import time
from threading import Thread

now_ms = lambda: int(round(time.time() * 1000))

max_loop = 40000
chunk_size = 1000
loop = 0
the_threads = []

start = now_ms()

def async(chunk):
    loop = 0
    while( loop < chunk ):
        url = 'http://localhost:9000/api/v1/version'
        # let it throw exception on fail
        response = urllib2.urlopen(url).read()
        loop = loop + 1


while loop < max_loop :
    thread = Thread( target = async, args = (chunk_size,) )
    the_threads.append(thread)
    thread.start()
    loop = loop + chunk_size

print "Waiting..."
# wait for all
while len(the_threads):
    wait_for_me = the_threads.pop();
    wait_for_me.join()

end = now_ms()

ms = end - start
print( str(max_loop) + " transactions in " + str(ms) + " milliseconds.")
seconds = ms / 1000.00000
tps = max_loop / seconds
print( "Observed TPS is about " + str(tps) )









__author__ = 'lpreimesberger'

import ecdsa
import base64
import hashlib
from ecdsa.util import string_to_number

# secp256k1, http://www.oid-info.com/get/1.3.132.0.10
_p = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFC2FL
_r = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEBAAEDCE6AF48A03BBFD25E8CD0364141L
_b = 0x0000000000000000000000000000000000000000000000000000000000000007L
_a = 0x0000000000000000000000000000000000000000000000000000000000000000L
_Gx = 0x79BE667EF9DCBBAC55A06295CE870B07029BFCDB2DCE28D959F2815B16F81798L
_Gy = 0x483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8L
curve_secp256k1 = ecdsa.ellipticcurve.CurveFp( _p, _a, _b )
generator_secp256k1 = ecdsa.ellipticcurve.Point( curve_secp256k1, _Gx, _Gy, _r )
oid_secp256k1 = (1,3,132,0,10)
SECP256k1 = ecdsa.curves.Curve("SECP256k1", curve_secp256k1, generator_secp256k1, oid_secp256k1 )

addrtype = 0

# from http://eli.thegreenplace.net/2009/03/07/computing-modular-square-roots-in-python/
def modular_sqrt(a, p):
    """ Find a quadratic residue (mod p) of 'a'. p
    must be an odd prime.

    Solve the congruence of the form:
    x^2 = a (mod p)
    And returns x. Note that p - x is also a root.

    0 is returned is no square root exists for
    these a and p.

    The Tonelli-Shanks algorithm is used (except
    for some simple cases in which the solution
    is known from an identity). This algorithm
    runs in polynomial time (unless the
    generalized Riemann hypothesis is false).
    """
    # Simple cases
    #
    if legendre_symbol(a, p) != 1:
        return 0
    elif a == 0:
        return 0
    elif p == 2:
        return p
    elif p % 4 == 3:
        return pow(a, (p + 1) / 4, p)

    # Partition p-1 to s * 2^e for an odd s (i.e.
    # reduce all the powers of 2 from p-1)
    #
    s = p - 1
    e = 0
    while s % 2 == 0:
        s /= 2
        e += 1

    # Find some 'n' with a legendre symbol n|p = -1.
    # Shouldn't take long.
    #
    n = 2
    while legendre_symbol(n, p) != -1:
        n += 1

    # Here be dragons!
    # Read the paper "Square roots from 1; 24, 51,
    # 10 to Dan Shanks" by Ezra Brown for more
    # information
    #

    # x is a guess of the square root that gets better
    # with each iteration.
    # b is the "fudge factor" - by how much we're off
    # with the guess. The invariant x^2 = ab (mod p)
    # is maintained throughout the loop.
    # g is used for successive powers of n to update
    # both a and b
    # r is the exponent - decreases with each update
    #
    x = pow(a, (s + 1) / 2, p)
    b = pow(a, s, p)
    g = pow(n, s, p)
    r = e

    while True:
        t = b
        m = 0
        for m in xrange(r):
            if t == 1:
                break
            t = pow(t, 2, p)

        if m == 0:
            return x

        gs = pow(g, 2 ** (r - m - 1), p)
        g = (gs * gs) % p
        x = (x * gs) % p
        b = (b * g) % p
        r = m

def legendre_symbol(a, p):
    """ Compute the Legendre symbol a|p using
    Euler's criterion. p is a prime, a is
    relatively prime to p (if p divides
    a, then a|p = 0)

    Returns 1 if a has a square root modulo
    p, -1 otherwise.
    """
    ls = pow(a, (p - 1) / 2, p)
    return -1 if ls == p - 1 else ls

__b58chars = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
__b58base = len(__b58chars)

def b58encode(v):
    """ encode v, which is a string of bytes, to base58.
    """

    long_value = 0L
    for (i, c) in enumerate(v[::-1]):
        long_value += (256**i) * ord(c)

    result = ''
    while long_value >= __b58base:
        div, mod = divmod(long_value, __b58base)
        result = __b58chars[mod] + result
        long_value = div
    result = __b58chars[long_value] + result

    # Bitcoin does a little leading-zero-compression:
    # leading 0-bytes in the input become leading-1s
    nPad = 0
    for c in v:
        if c == '\0': nPad += 1
        else: break

    return (__b58chars[0]*nPad) + result

def msg_magic(message):
    return message;
    return "\x18Bitcoin Signed Message:\n" + chr( len(message) ) + message

def Hash(data):
    return hashlib.sha256(hashlib.sha256(data).digest()).digest()

def hash_160(public_key):
    md = hashlib.new('ripemd160')
    md.update(hashlib.sha256(public_key).digest())
    return md.digest()

def hash_160_to_bc_address(h160):
    vh160 = chr(addrtype) + h160
    h = Hash(vh160)
    addr = vh160 + h[0:4]
    return b58encode(addr)

def public_key_to_bc_address(public_key):
    h160 = hash_160(public_key)
    return hash_160_to_bc_address(h160)

def encode_point(pubkey, compressed=False):
    order = generator_secp256k1.order()
    p = pubkey.pubkey.point
    x_str = ecdsa.util.number_to_string(p.x(), order)
    y_str = ecdsa.util.number_to_string(p.y(), order)
    if compressed:
        return chr(2 + (p.y() & 1)) + x_str
    else:
        return chr(4) + x_str + y_str

def sign_message(private_key, message, compressed=False):
    public_key = private_key.get_verifying_key()
    signature = private_key.sign_digest( Hash( msg_magic( message ) ), sigencode = ecdsa.util.sigencode_string )
    address = public_key_to_bc_address(encode_point(public_key, compressed))
    assert public_key.verify_digest( signature, Hash( msg_magic( message ) ), sigdecode = ecdsa.util.sigdecode_string)
    for i in range(4):
        nV = 27 + i
        if compressed:
            nV += 4
        sig = base64.b64encode( chr(nV) + signature )
        try:
            if verify_message( address, sig, message):
                return sig
        except:
            continue
    else:
        raise BaseException("error: cannot sign message")

def verify_message(address, signature, message):
    """ See http://www.secg.org/download/aid-780/sec1-v2.pdf for the math """
    from ecdsa import numbertheory, ellipticcurve, util
    curve = curve_secp256k1
    G = generator_secp256k1
    order = G.order()
    # extract r,s from signature
    sig = base64.b64decode(signature)
    if len(sig) != 65: raise BaseException("Wrong encoding")
    r,s = util.sigdecode_string(sig[1:], order)
    nV = ord(sig[0])
    if nV < 27 or nV >= 35:
        return False
    if nV >= 31:
        compressed = True
        nV -= 4
    else:
        compressed = False
    recid = nV - 27
    # 1.1
    x = r + (recid/2) * order
    # 1.3
    alpha = ( x * x * x  + curve.a() * x + curve.b() ) % curve.p()
    beta = modular_sqrt(alpha, curve.p())
    y = beta if (beta - recid) % 2 == 0 else curve.p() - beta
    # 1.4 the constructor checks that nR is at infinity
    R = ellipticcurve.Point(curve, x, y, order)
    # 1.5 compute e from message:
    h = Hash( msg_magic( message ) )
    e = string_to_number(h)
    minus_e = -e % order
    # 1.6 compute Q = r^-1 (sR - eG)
    inv_r = numbertheory.inverse_mod(r,order)
    Q = inv_r * ( s * R + minus_e * G )
    public_key = ecdsa.VerifyingKey.from_public_point( Q, curve = SECP256k1 )
    # check that Q is the public key
    public_key.verify_digest( sig[1:], h, sigdecode = ecdsa.util.sigdecode_string)
    # check that we get the original signing address
    addr = public_key_to_bc_address(encode_point(public_key, compressed))
    if address == addr:
        return True
    else:
        #print addr
        return False

if __name__ == '__main__':
    # some simple testing code
    print verify_message('16vqGo3KRKE9kTsTZxKoJKLzwZGTodK3ce',
            'HPDs1TesA48a9up4QORIuub67VHBM37X66skAYz0Esg23gdfMuCTYDFORc6XGpKZ2/flJ2h/DUF569FJxGoVZ50=',
            'test message') # good
    print verify_message('16vqGo3KRKE9kTsTZxKoJKLzwZGTodK3ce',
            'HPDs1TesA48a9up4QORIuub67VHBM37X66skAYz0Esg23gdfMuCTYDFORc6XGpKZ2/flJ2h/DUF569FJxGoVZ50=',
            'test message 2') # bad

    private_key = ecdsa.SigningKey.from_string( '5JkuZ6GLsMWBKcDWa5QiD15Uj467phPR', curve = SECP256k1 )
    public_key = private_key.get_verifying_key()
    bitcoinaddress = public_key_to_bc_address( encode_point(public_key) )
    print bitcoinaddress
    sig = sign_message(private_key, 'test message')
    print sig
    print verify_message(bitcoinaddress, sig, 'test message')
    print verify_message('1GdKjTSg2eMyeVvPV5Nivo6kR8yP2GT7wF',
            'GyMn9AdYeZIPWLVCiAblOOG18Qqy4fFaqjg5rjH6QT5tNiUXLS6T2o7iuWkV1gc4DbEWvyi8yJ8FvSkmEs3voWE=',
            'freenode:#bitcoin-otc:b42f7e7ea336db4109df6badc05c6b3ea8bfaa13575b51631c5178a7')

    print verify_message('1Hpj6xv9AzaaXjPPisQrdAD2tu84cnPv3f',
            'INEJxQnSu6mwGnLs0E8eirl5g+0cAC9D5M7hALHD9sK0XQ66CH9mas06gNoIX7K1NKTLaj3MzVe8z3pt6apGJ34=',
            'testtest')
    print verify_message('18uitB5ARAhyxmkN2Sa9TbEuoGN1he83BX',
            'IMAtT1SjRyP6bz6vm5tKDTTTNYS6D8w2RQQyKD3VGPq2i2txGd2ar18L8/nvF1+kAMo5tNc4x0xAOGP0HRjKLjc=',
            'testtest')

    # sign compressed key
    compressed = True
    secret = 'dea7715ddcf5aba27530d6a1393813fbdd09af3aeb5f4f1616f563833d07babb'.decode('hex')
    private_key = ecdsa.SigningKey.from_string( secret, curve = SECP256k1 )
    public_key = private_key.get_verifying_key()
    bitcoinaddress = public_key_to_bc_address( encode_point(public_key, compressed) )
    print bitcoinaddress

    sig = sign_message(private_key, 'test message', compressed)
    print sig

    print verify_message(bitcoinaddress, sig, 'test message')

    print verify_message('1LsPb3D1o1Z7CzEt1kv5QVxErfqzXxaZXv',
            'H3I37ur48/fn52ZvWQT+Mj2wXL36gyjfaN5qcgfiVRTJb1eP1li/IacCQspYnUntiRv8r6GDfJYsdiQ5VzlG3As=',
            'testtest')



__author__ = 'census protocol foundation'

import urllib2
import time
import json
import pybitcointools
import requests


headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
now_ms = lambda: int(round(time.time() * 1000))
serialize = lambda json_data: json.dumps( json_data, sort_keys=True, separators=(',',':') )

def generate_wallet( my_seed_data ):
    """ create a new empty wallet - does not post it.  this is a VERY EXPENSIVE operation
    :param my_seed_data: seed for the wallet - this should be a big random block, but as always...
    :return: object containing, public, private, and address info
    """
    new_wallet = {}
    hash_of_seed = pybitcointools.sha256(my_seed_data)
    new_wallet["public"] = pybitcointools.privkey_to_pubkey(hash_of_seed)
    new_wallet["private"] = hash_of_seed;
    new_wallet["address"] = pybitcointools.pubtoaddr( new_wallet["public"])
    return new_wallet

def version( host ):
    """ get the version block from the given server
    :param host: host to connect to
    :return: json data for that host or raises exception
    """
    url = 'http://{}:9000/api/v1/version'.format(host)
    # let it throw exception on fail and bubble up
    return urllib2.urlopen(url).read()


def sign( json_data, private_key ):
    """ sign the json info - returning the json object modified
    :param json_data: what to sign
    :param private_key: ecdsa key to use
    :return: updated json_data.  signature replaced with new signature
    """
    # clear if present
    json_data['signature'] = ""
    # sort and normalize the json data
    theString = json.dumps( json_data, sort_keys=True, separators=(',',':') )
    print "signing\n" + theString
    signature = pybitcointools.ecdsa_sign( theString, private_key )
    json_data['signature'] = signature
    print "signed!!!"
    print json_data
    print "signed!!!"
    return json_data


def check( json_data, public_key ):
    """ implementing the bitcoin.message.sign using the address is horrible in python
    i linked in a tiny library in crypto from electrum - but the bitcoin way assumes you
    only can add 255 bytes to the message since there's a pascal-style string at the front.
    this requires a wallet PUBLIC KEY - the web client is happy with the address
    :param json_data: data to check
    :param public_key: ecdsa encoded PUBLIC KEY - not the address
    :return: true or false
    """
    # clear if present
    the_signature = json_data['signature']
    json_data['signature'] = ""
    # sort and normalize the json data
    theString = json.dumps( json_data, sort_keys=True, separators=(',',':') )
    is_okay = pybitcointools.ecdsa_verify( theString, the_signature, public_key )
    return is_okay


def vend( host, json_data ):
    """ send this json to the host as a vend transaction
    :param host: host to connect to
    :param json_data: data to send.  must be signed
    :return: server response - normally json
    """
    r = requests.post(
        "http://{}:9000/api/v1/vend".format(host),
        headers=headers,
        data=serialize(json_data)
    )
    return r.text

def item( host, json_data ):
    """ send this json to the host as a item transaction
    :param host: host to connect to
    :param json_data: data to send.  must be signed
    :return: server response - normally json
    """
    r = requests.post(
        "http://{}:9000/api/v1/item".format(host),
        headers=headers,
        data=serialize(json_data)
    )
    return r.text

def items_from_venue( host, venue ):
    """ find items from this venue - capped, but at least 50
    :param host: host to connect to
    :param json_data: data to send.  must be signed
    :return: server response - normally json
    """
    r = requests.get(
        "http://{}:9000/api/v1/item/venue/{}".format(host, venue),
        headers=headers

    )
    return r.text

def items_from_location( host, lat, lon, distance_km ):
    """ get items around a certain point
    :param host:
    :param lat: latitude
    :param lon: longitude
    :param distance_km: how far away in km
    :return: listing
    """
    r = requests.get(
        "http://{}:9000/api/v1/item/geo/{}/{}/{}".format(host, lat, lon, distance_km),
        headers=headers

    )
    return r.text

def get_item( host, txid ):
    """ returns current state for this item
    :param host: host to connect to
    :param txid: data to send.  must be signed
    :return: server response - normally json
    """
    r = requests.get(
        "http://{}:9000/api/v1/item/{}".format(host, txid ),
        headers=headers

    )
    return r.text



def buy( host, json_data ):
    """ send this json to the host as a buy transaction
    :param host: host to connect to
    :param json_data: data to send.  must be signed
    :return: server response - normally json
    """
    r = requests.post(
        "http://{}:9000/api/v1/buy".format(host),
        headers=headers,
        data=serialize(json_data)
    )
    return r.text


def wallet( host, wallet_id ):
    """ send this json to the host as a wallet create transaction
    :param host: host to connect to
    :param wallet_id: the wallet uuid
    :return: server response - normally json
    """
    r = requests.post(
        "http://{}:9000/api/v1/wallet/{}".format(host. wallet_id ),
        headers=headers,
        data=serialize(json_data)
    )
    return r.text

def get_wallet( host, wallet_id ):
    """ get the information for a given wallet
    :param host: host to connect to
    :param wallet_id: the wallet uuid
    :return: server response - normally json
    """
    r = requests.get(
        "http://{}:9000/api/v1/wallet/{}".format(host, wallet_id),
        headers=headers
    )
    return r.text

def get_owned_items( host, wallet_id ):
    """ get the items owned by this user
    :param host: host to connect to
    :param wallet_id: the wallet uuid
    :return: server response - normally json
    """
    r = requests.get(
        "http://{}:9000/api/v1/wallet/owned/{}".format(host, wallet_id),
        headers=headers
    )
    return r.text



__author__ = 'lpreimesberger'
__copyright__ = 'copyright (c) 2015 census protocol foundation'
#
#
# this is part of the test suite - you are free to use  the software for any  purpose
# we welcome your donations - all are tax deductible and  go to education and pay for
# core  protocol  developers and  evangelists.  the sitting board receives no salary.

#  "You  have  to  find  a  way to earn your living without transgressing your ideals
#   of love and compassion" - Thich Nhat Hanh

#                          19MSTA2Zcuga7rTqLY5vfXLWDBkdfcVQ2U
#
#  if you require a tax receipt for your donation - send the txid to donate@censusprotocol.org
#
#
# The MIT License (MIT)
#
# Copyright (c) 2015 Census Protocol Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining a  copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the  rights
# to use, copy, modify, merge, publish,  distribute,  sublicense,  and/or  sell
# copies  of  the  Software,  and  to  permit persons to whom the  Software  is
# furnished to do so, subject to the following conditions:
#
# The  above copyright notice  and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE  SOFTWARE  IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING  BUT  NOT LIMITED  TO  THE  WARRANTIES OF MERCHANTABILITY,
# FITNESS  FOR  A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS   OR  COPYRIGHT  HOLDERS  BE LIABLE  FOR ANY  CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR  OTHERWISE, ARISING FROM,
# OUT OF  OR  IN  CONNECTION WITH THE SOFTWARE OR THE USE  OR OTHER DEALINGS IN
# THE SOFTWARE.


import sys
import time
import uuid
import json
import pybitcointools
import requests

headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}

from threading import Thread
# import the constants for this release
with open( '../server/server/census/v01-00.json') as data_file:
    census = json.load(data_file)
# python has a weird cfg format, using same as everything else
with open( 'config.json') as data_file:
    config = json.load(data_file)

json_data = {"version": 1,
               "item_type": "buy",
               "txid": "1",
               "created":"1",
               "block_in": "",
               "vend_item":"x",
               "block_in": "",
               "price":2000,
               "source": config['NODEWALLET'], #str(new_key.address),
               "target": config['NODEWALLET'],
               "expiration": "11/11/2015 20:00:00",
               "tips": 10,
               "signature": ""}

if len(sys.argv) != 2:
    print "Usage\npython vend_test.py <txid to sell>"
    sys,exit()

print sys.argv[1]
now_ms = lambda: int(round(time.time() * 1000))

max_loop = 4000
chunk_size = 1000
loop = 0
the_threads = []

start = now_ms()

def async(chunk):
    loop = 0
    while( loop < chunk ):
        json_data["signature"] = "";
        json_data["txid"] = str(uuid.uuid4())
        json_data["expiration"] = now_ms()+360000000;
        json_data["created"] = now_ms()+0;
        json_data["vend_item"] = sys.argv[1]
        # compute signature for the current key
        theString = json.dumps( json_data, sort_keys=True, separators=(',',':') )
        print "signing with [" + config['RAWPKEY'] + "]"
        signature = pybitcointools.ecdsa_sign( theString, config['RAWPKEY'] ) # new_key.pk )
        json_data["signature"] = str(signature)
        print json_data
        r = requests.post(
            "http://localhost:9000/api/v1/buy",
            headers=headers,
            data=json.dumps( json_data, sort_keys=True, separators=(',',':') )
        )
        print r.text
        loop = loop + 1


#while loop < max_loop :
#    thread = Thread( target = async, args = (chunk_size,) )
#    the_threads.append(thread)
#    thread.start()
#    loop = loop + chunk_size

#print "Waiting..."
# wait for all
#while len(the_threads):
#    wait_for_me = the_threads.pop();
#    wait_for_me.join()

async(1)
end = now_ms()

ms = end - start
print( str(max_loop) + " transactions in " + str(ms) + " milliseconds.")
seconds = ms / 1000.00000
tps = max_loop / seconds
print( "Observed TPS is about " + str(tps) )






